@@ -5,11 +5,10 @@ import sys
 import time
 
 from blazarclient.exception import BlazarClientException
-from neutronclient.v2_0.client import Client as NeutronClient
 
-from .clients import blazar
+from .clients import blazar, neutron
 from .context import get as get_from_context, session
-from .network import get_network_id, PUBLIC_NETWORK
+from .network import get_network_id, PUBLIC_NETWORK, list_floating_ips
 from .server import Server, ServerError
 from .util import random_base32, utcnow
 
@@ -20,7 +19,8 @@ __all__ = [
     'add_node_reservation',
     'add_network_reservation',
     'add_fip_reservation',
-    'get_floating_ip_by_reservation_id',
+    'get_node_reservation',
+    'get_reserved_floating_ips',
     'lease_duration',
     'get_lease',
     'get_lease_id',
@@ -187,10 +187,8 @@ class Lease(object):
         kwargs.setdefault("session", session())
 
         self.session = kwargs.pop("session")
-        self.blazar = BlazarClient(
-            "1", service_type="reservation", session=self.session
-        )
-        self.neutron = NeutronClient(session=self.session)
+        self.blazar = blazar(session=self.session)
+        self.neutron = neutron(session=self.session)
 
         self.lease = None
 
@@ -388,16 +386,7 @@ def get_node_reservation(lease_ref, count=None, node_type=None):
     Raises:
         ValueError: If no reservation was found, or multiple were found.
     """
-    lease = get_lease(lease_ref)
-    reservations = lease.get("reservations", [])
-    if isinstance(reservations, str):
-        LOG.info("Blazar returned nested JSON structure, unpacking.")
-        try:
-            reservations = json.loads(reservations)
-        except Exception:
-            pass
-
-    def _passes(res):
+    def _find_node_reservation(res):
         if res.get("resource_type") != "physical:host":
             return False
         if (count is not None
@@ -409,15 +398,52 @@ def get_node_reservation(lease_ref, count=None, node_type=None):
             return False
         return True
 
-    node_reservations = [r for r in reservations if _passes(r)]
-    if not node_reservations:
-        raise ValueError("No node reservation found")
-    elif len(node_reservations) > 1:
-        raise ValueError("Multiple node reservations found")
-    else:
-        return node_reservations[0]
+    res = _reservation_matching(lease_ref, _find_node_reservation)
+    return res["id"]
 
 
+def get_reserved_floating_ips(lease_ref) -> "list[str]":
+    """Get a list of Floating IP addresses reserved in a lease.
+
+    Args:
+        lease_ref (str): The ID or name of the lease.
+
+    Returns:
+        A list of all reserved Floating IP addresses, if any were reserved.
+    """
+    def _find_fip_reservation(res):
+        return res.get("resource_type") == "virtual:floatingip"
+
+    res = _reservation_matching(lease_ref, _find_fip_reservation, multiple=True)
+    fips = list_floating_ips()
+    return [
+        fip["floating_ip_address"] for fip in fips
+        if any(f"reservation:{r['id']}" in fip["tags"] for r in res)
+    ]
+
+
+def _reservation_matching(lease_ref, match_fn, multiple=False):
+    lease = get_lease(lease_ref)
+    reservations = lease.get("reservations", [])
+    if isinstance(reservations, str):
+        LOG.info("Blazar returned nested JSON structure, unpacking.")
+        try:
+            reservations = json.loads(reservations)
+        except Exception:
+            pass
+
+    matches = [r for r in reservations if match_fn(r)]
+
+    if not matches:
+        raise ValueError("No matching reservation found")
+
+    if multiple:
+        return matches
+    else:
+        if len(matches) > 1:
+            raise ValueError("Multiple matching reservations found")
+        return matches[0]
+
 
 def add_network_reservation(reservation_list,
                             network_name,
@@ -475,11 +501,6 @@ def add_fip_reservation(reservation_list, count=1):
     })
 
 
-def get_floating_ip_by_reservation_id(reservation_id):
-    # blazar().lease.list()
-    pass
-
-
 def lease_duration(days=1, hours=0):
     """Compute the start and end dates for a lease given its desired duration.
 
@@ -596,15 +617,18 @@ def wait_for_active(ref):
     Args:
         ref (str): The name or ID of the lease.
 
+    Returns:
+        The lease in ACTIVE state.
+
     Raises:
         TimeoutError: If the lease fails to become active within the timeout.
     """
     for _ in range(15):
-        time.sleep(10)
-        status = get_lease(ref)['status']
+        lease = get_lease(ref)
+        status = lease['status']
         if status == 'ACTIVE':
-            break
+            return lease
         elif status == 'ERROR':
             raise RuntimeError("Lease went into ERROR state")
-    else:
-        raise TimeoutError("Lease failed to start")
+        time.sleep(10)
+    raise TimeoutError("Lease failed to start")
